!
! dirCosRotMatEl --- representation of the matrix of direction cosines
!       between the body-fixed frame and the space-fixed frame in terms
!       of symmetric rotor eigenfunctions
! Christian Buth
! 16 Apr 2007
!
! Version 1.0.0: 16 Apr 2007 (Moved functions from the main file alignmol
!       to this module)
! Version 1.1.0: 13 Jul 2007 (Removed bug from direction cosine
!       matrix elements: wrong handling of zero angular momenta)
!
! The name Kroto refers to the book:
! H. W. Kroto, Molecular rotation spectra, John Wiley & Sons,
! London, 1975, ISBN: 0-471-50853-5
!


      module dirCosRotMatEl
      use fortrantl
      implicit none

      contains

!
! reduced_S --- compute the reduced matrix element of the matrix S
!       in symmetric rotor eigenfunctions.  Kroto, page 76
!

      real(dblprec) function reduced_S(J1, J2)
      implicit none

      integer J1, J2
      real(dblprec) rJ1

!       if (J1 <= 0 .or. J2 <= 0) then
      if (J1 < 0 .or. J2 < 0) then
        reduced_S = 0.0D0
        return
      end if

      rJ1 = J1

      if (J1 == J2 .and. J1 /= 0) then
        reduced_S = 1.0D0 / (rJ1 * (rJ1+1.0D0))
      else if (J1+1 == J2) then
        reduced_S = 1.0D0 / ((rJ1+1.0D0) * sqrt((2.0D0 * rJ1 + 1.0D0) * (2.0D0 * rJ1 + 3.0D0)))
      else if (J1-1 == J2) then
        reduced_S = 1.0D0 / ( rJ1        * sqrt((2.0D0 * rJ1 + 1.0D0) * (2.0D0 * rJ1 - 1.0D0)))
      else
        reduced_S = 0.0D0
      end if

!       if ((J1 == 0 .or. J2 == 0) .and. reduced_S /= 0.0D0) then
!          write (199,*) '(J1 == 0 .or. J2 == 0) .and. reduced_S /= 0.0D0', J1, J2, reduced_S
!       end if
      end function reduced_S


!
! lab_sx_S --- compute the matrix element of the s_x angular
!       momentum operator in the laboratory frame in terms of the
!       |J M> eigenfunctions to J^2 and J_z.
!

      real(dblprec) function lab_sx_S(J1, M1, J2, M2)
      implicit none

      integer J1, M1, J2, M2

      lab_sx_S = 0.5D0 * (lab_spm_S(J1, M1, J2, M2, +1) + lab_spm_S(J1, M1, J2, M2, -1))
      end function lab_sx_S


!
! lab_sy_S --- compute the matrix element of the s_y angular
!       momentum operator in the laboratory frame in terms of the
!       |J M> eigenfunctions to J^2 and J_z.
!

      complex(dblprec) function lab_sy_S(J1, M1, J2, M2)
      implicit none

      integer J1, M1, J2, M2

      lab_sy_S = 0.5D0 / (0.0D0,1.0D0) * cmplx(lab_spm_S(J1, M1, J2, M2, +1)  &
     &                                       - lab_spm_S(J1, M1, J2, M2, -1))
      end function lab_sy_S


!
! lab_spm_S --- compute the matrix element of the s_+- angular
!       momentum operator in the laboratory frame in terms of the
!       |J M> eigenfunctions to J^2 and J_z.  Kroto, page 76
!

      real(dblprec) function lab_spm_S(J1, M1, J2, M2, signum)
      implicit none

      integer J1, M1, J2, M2, signum
      real(dblprec) rJ1, rM1, rsig

      if (abs(signum) /= 1) then
        stop 'lab_spm_S: abs(signum) /= 1!'
      end if

      if (M2 /= (M1 - signum) .or. J1 < 0 .or. J2 < 0 .or.  &
     &    J1 < abs(M1) .or. J2 < abs(M2)) then
        lab_spm_S = 0.0D0
        return
      end if

      rJ1  = J1
      rM1  = M1
      rsig = signum

      if (J1 == J2) then
        lab_spm_S =           sqrt((rJ1 + rsig * rM1)         * (rJ1 - rsig * rM1 + 1.0D0))
      else if (J1+1 == J2) then
        lab_spm_S =   rsig  * sqrt((rJ1 - rsig * rM1 + 1.0D0) * (rJ1 - rsig * rM1 + 2.0D0))
      else if (J1-1 == J2) then
        lab_spm_S = (-rsig) * sqrt((rJ1 + rsig * rM1)         * (rJ1 + rsig * rM1 - 1.0D0))
      else
        lab_spm_S = 0.0D0
      end if

!       if ((J1 == 0 .or. J2 == 0) .and. lab_spm_S /= 0.0D0) then
!          write (199,*) '(J1 == 0 .or. J2 == 0) .and. lab_spm_S /= 0.0D0', J1, J2, lab_spm_S
!       end if
      end function lab_spm_S


!
! lab_sz_S --- compute the matrix element of the s_z angular
!       momentum operator in the laboratory frame in terms of the
!       |J M> eigenfunctions to J^2 and J_z.  Kroto, page 76
!

      real(dblprec) function lab_sz_S(J1, J2, M)
      implicit none

      integer J1, J2, M
      real(dblprec) rJ1, rM

!       if (J1 <= 0 .or. J2 <= 0 .or. J1 < abs(M) .or. J2 < abs(M)) then
      if (J1 < 0 .or. J2 < 0 .or. J1 < abs(M) .or. J2 < abs(M)) then
        lab_sz_S = 0.0D0
        return
      end if

      rJ1 = J1
      rM  = M

      if (J1 == J2) then
        lab_sz_S = rM
      else if (J1+1 == J2) then
        lab_sz_S = sqrt((rJ1 + rM + 1.0D0) * (rJ1 - rM + 1.0D0))
      else if (J1-1 == J2) then
        lab_sz_S = sqrt((rJ1 + rM        ) * (rJ1 - rM        ))
      else
        lab_sz_S = 0.0D0
      end if

!       if ((J1 == 0 .or. J2 == 0) .and. lab_sz_S /= 0.0D0) then
!          write (199,*) '(J1 == 0 .or. J2 == 0) .and. lab_sz_S /= 0.0D0', J1, J2, lab_sz_S
!       end if
      end function lab_sz_S


!
! mol_sa_S --- compute the matrix element of the s_a angular
!       momentum operator in the molecule-fixed frame in terms of the
!       |J K> eigenfunctions to J^2 and J_c.
!

      real(dblprec) function mol_sa_S(J1, K1, J2, K2)
      implicit none

      integer J1, K1, J2, K2

      mol_sa_S = 0.5D0 * (mol_spm_S(J1, K1, J2, K2, +1) + mol_spm_S(J1, K1, J2, K2, -1))
      end function mol_sa_S


!
! mol_sb_S --- compute the matrix element of the s_b angular
!       momentum operator in the molecule-fixed frame in terms of the
!       |J K> eigenfunctions to J^2 and J_c.
!

      complex(dblprec) function mol_sb_S(J1, K1, J2, K2)
      implicit none

      integer J1, K1, J2, K2

      mol_sb_S = 0.5D0 / (0.0D0,1.0D0) * cmplx(mol_spm_S(J1, K1, J2, K2, +1)  &
     &                                       - mol_spm_S(J1, K1, J2, K2, -1))
      end function mol_sb_S


!
! mol_spm_S --- compute the matrix element of the s_+- angular
!       momentum operator in the molecule-fixed frame in terms of the
!       |J K> eigenfunctions to J^2 and J_c.  Kroto, page 76
!

      real(dblprec) function mol_spm_S(J1, K1, J2, K2, signum)
      implicit none

      integer J1, K1, J2, K2, signum

!       mol_spm_S = lab_spm_S(J1, M1, J2, M2, signum)
      mol_spm_S = lab_spm_S(J1, K1, J2, K2, -signum)
      end function mol_spm_S


!
! mol_sc_S --- compute the matrix element of the s_c angular
!       momentum operator in the molecule-fixed frame in terms of the
!       |J K> eigenfunctions to J^2 and J_c.  Kroto, page 76
!

      real(dblprec) function mol_sc_S(J1, J2, K)
      implicit none

      integer J1, J2, K

      mol_sc_S = lab_sz_S(J1, J2, K)
      end function mol_sc_S
      end module dirCosRotMatEl
