!
! dreyd --- dressed Rydberg calculation
! Robin Santra, Christian Buth
! 5 June 2006
!
! Version V1.0.0: 23 Jun 2006 (Namelist support added)
! Version V1.1.0: 10 Nov 2006 (Transition matrix elements with
!       the Lanczos algorithm)
! Version V1.2.0: (The velocity form of the interaction Hamiltonian
!       with the x rays added.  New treatment of atomic polarizabilies)
! Version V1.3.0: (Corrected A^2-term; divided it by 4)
!


      program dreyd
      use mathtl
      use physconst
      use floquetmat
      implicit none

      interface
      subroutine lanczostransfloquet(nmat, nmain, niter, ceig, cpolest, nvec, nl,  &
     &          nphmin, lmin, coredip, normsq, jnd)
      use floquetmat
      use sparse
      implicit none

      integer, intent(in) :: nmat, nmain, niter, nvec, nl, nphmin, lmin
      integer, intent(out) :: jnd
      complex(dblprec), intent(in) :: coredip(nmat)
      complex(dblprec), intent(out) :: normsq, ceig(niter), cpolest(niter)
      end subroutine lanczostransfloquet
      end interface

      ! Use full (.true.) or iterative diagonalization (.false.)
      logical fulldiag
      ! Use the length form or the velocity form to evaluate the interaction
      ! with the x rays
      logical lengthform
      ! Number of x-ray photon energy grid points desired
      integer nx
      ! Orbital angular momentum quantum numbers for the
      ! initial one-particle state
      integer li
      ! Orbital angular momentum quantum numbers for the
      ! final one-particle state
      integer lf, mf
      ! Range of orbital angular momenta taken into account
      ! in Floquet calculation; s, p, and d orbitals should
      ! always be included
      integer lmin, lmax
      ! Occupied orbitals in the s: nocc(0), p: nocc(1), ... shells
      integer, parameter :: maxlocc = 50
      integer nocc(0:maxlocc), nshellocc(0:maxlocc)
      ! Number of l blocks
      integer nl
      ! Range of photon blocks taken into account
      integer nphmin, nphmax
      ! Number of photon blocks
      integer nph
      ! Number of atomic eigenvectors per l and photon block
      integer nvec
      ! Number of angle to be explored in a polar plot of the
      ! total cross section
      integer npolar
      ! Atomic number of the laser-dressed atom
      integer atmnumZ
      ! Energy at which the polar plot is to be made
      real(dblprec) Epolar
      ! Start and stop angle for the polar plot
      real(dblprec) angleBeg, angleEnd

      integer dimen, nmain, niter, nvlnc, i, j, k, l, init, jnd
      integer, parameter :: uxsect = 97,  &     ! Energy dependent cross section file
     &          upolar = 98                     ! Polar plot of the cross section

      real(dblprec) omegaX, E1s, Gamma1s, Xmin, Xmax, Delta, polarization, &
     &          sigtot, thetaLX, omegaL, intens, cconst, Eground
      real(dblprec), allocatable :: eval(:,:)

      complex(dblprec), allocatable :: coredip(:),  &
     &          lanczstartvec(:),                   &
     &          SESWatom(:,:,:),                    &
     &          SESdipatm(:,:,:,:),                 &
     &          SESpotatm(:,:,:),                   &
     &          SESfTfatm(:,:,:),                   &
     &          spectrum(:,:),                      &
     &          evec(:,:),                          &
     &          trans(:,:),                         &
     &          polestrength(:,:),                  &
     &          normsq(:)

      namelist/DressedRydberg/ nx, li, lf, lmax, nphmin,             &
     &          nphmax, nvec, nocc, omegaL, intens, thetaLX, Xmin,   &
     &          Xmax, E1s, Gamma1s, init, fulldiag, npolar, Epolar,  &
     &          angleBeg, angleEnd, niter, lengthform

      ! Initialization
      write (*,*) 'dreyd V1.3.0, dressed Rydberg calculations'
      write (*,*) '(c) 2006 Christian Buth, Robin Santra'
      write (*,*)

      ! Default values for parameters
      nx = 10001
      li = 0 ; lf = 1
      lmin = 0 ; lmax = 2
      nl = lmax - lmin + 1
      nphmin = -2 ; nphmax = 2
      nph = nphmax - nphmin + 1
      nvec = 50
      npolar = 0
      Epolar = 0.0D0
      angleBeg = 0.0D0 ; angleEnd = 0.0D0
      nocc(:) = 0
      omegaL = 1.5498D0
      intens = 1.0D13
      thetaLX = 0.0D0
      Xmin = 14310.0D0
      Xmax = 16410.0D0
      E1s = -14326.0D0
      Gamma1s = 2.7D0
      init = 1
      fulldiag = .true.
      lengthform = .true.
      niter = 100
      nmain = 1

      read(*,nml=DressedRydberg)

      nl = lmax - lmin + 1
      nph = nphmax - nphmin + 1
      if (lmin < 0 .or. lmin > maxlocc .or. lmax < 0 .or. lmax > maxlocc) then
        write (*,*)
        write (*,*) 'dreyd: illegal parameter range for lmin and lmax'
        write (*,*) 'Range of orbital angular momenta taken into account ', &
     &              'in Floquet calculation;'
        write (*,*) '    s, p, and d orbitals should always be included: lmin =', &
     &              lmin, 'lmax =', lmax
        stop 'dreyd: illegal parameter range'
      end if

      ! Echo input
      write (*,*) 'Orbital angular momentum quantum numbers for the ', &
     &            'final   one-particle state: lf =', lf
      if (lf /= 1) then
        stop 'dreyd: code only for lf = 1 implemented!'
      end if
      write (*,*) 'Range of orbital angular momenta taken into account ', &
     &            'in Floquet calculation;'
      write (*,*) '    s, p, and d orbitals should always be included: lmin =', &
     &            lmin, 'lmax =', lmax
      atmnumZ = 0
      do j=lmin, lmax
        atmnumZ = atmnumZ + 2 * (2*j+1) * nocc(j)
      end do
      write (*,*) 'Atomic number of the atom', atmnumZ
      write (*,*) 'Number of orbitals occupied in the ground state:'
      write (*,*) 'nocc =', (nocc(j), j=lmin, lmax)
      write (*,*) 'Number of l blocks: nl =', nl
      write (*,*) 'Range of photon blocks taken into account: nphmin =', nphmin, &
     &            'nphmax =', nphmax
      write (*,*) 'Number of photon blocks: nph =', nph
      write (*,*) 'Number of atomic eigenvectors per l and photon block: nvec =', nvec

      dimen = nvec * nl * nph
      write (*,*) 'Dimension of Floquet matrix = ', dimen

      write (*,*) 'Laser photon energy in eV', omegaL
      write (*,*) 'Laser intensity in W/cm^2', intens
      write (*,*) 'Examine the photoabsorption of state', init       !
      write (*,*) 'Experimental (negative) binding energy of the electron in ',  &
     &            'the state in eV', E1s
      write (*,*) 'Experimental decay width of the hole in eV', Gamma1s
      write (*,*) 'Use full (.true.) or iterative diagonalization (.false.)', fulldiag
      if (.not. fulldiag) then
        write (*,*) 'Carrying out ', niter, ' band-Lanczos iterations',  &
     &              'with ', nmain, 'Lanczos vectors'
      else
        niter = dimen
      end if
      write (*,*) 'Use the length form (.true.) or the velocity form (.false.)', lengthform

      write (*,*) 'Energy plot:'
      write (*,*) 'Number of x-ray photon energy grid points: nx =', nx
      write (*,*) 'Angle in degrees between the polarizations of laser ',  &
     &            'and x-rays', thetaLX
      write (*,*) 'Desired x-ray photon energy range is from Xmin (in eV)', Xmin,  &
     &            'to Xmax (in eV)', Xmax

      write (*,*) 'Polar plot:'
      write (*,*) 'Number of angular grid points: npolar =', npolar
      write (*,*) 'X-ray photon energy (in eV)', Epolar
      write (*,*) 'Desired angular range is from angleBeg (in degree)', angleBeg,  &
     &            'to angleEnd (in degree)', angleEnd

      allocate(eval(nvec,lmin:lmax))

      allocate(coredip(nvec),                              &
     &          SESWatom(nvec,nvec,lmin:lmax),             &
     &          SESdipatm(nvec,lmin:lmax,nvec,lmin:lmax),  &
     &          SESpotatm(nvec,nvec,lmin:lmax),            &
     &          SESfTfatm(nvec,nvec,lmin:lmax))

      allocate(spectrum(niter,-lf:lf),       &
     &          polestrength(niter,-lf:lf),  &
     &          trans(niter,-lf:lf),         &
     &          normsq(-lf:lf))

      !ccccc laser parameters cccccccccccccccccccccccccccccccccccccccc
      ! omegaL: laser photon energy in eV:
      ! (800 nm = 1.5498 eV; 790 nm = 1.5694 eV; 810 nm = 1.5307 eV;
      ! 1092 nm = 1.1353 eV)
      ! conversion to Hartree:
      omegaL = omegaL * eVToHartree
      ! intens: laser intensity in W/cm^2
      ! conversion to atomic units (an atomic unit of intensity
      ! defined as hartree/t0/a0^2 = 6.43641d15 W/cm^2)
      intens = intens * intensity_SItoAU
      ! calculation of the coupling constant sqrt(2 pi alpha intens)
      cconst = sqrt(2.0D0 * M_PI * fine_structure_constant * intens)
      write (*,*) 'laser coupling constant in a.u. = ', cconst

      !ccccc x-ray parameters ccccccccccccccccccccccccccccccccccccccccc
      ! laser and x-rays are linearly polarized and co-propagating;
      ! thetaLX: angle between the polarizations of laser and x-rays;
      ! thetaLX should be entered in degrees
      ! conversion to rad
      thetaLX = thetaLX * degtorad
     ! the desired x-ray photon energy range is from Xmin (in eV)
     ! to Xmax (in eV); conversion to hartree
      Xmin = Xmin * eVToHartree
      Xmax = Xmax * eVToHartree
      Epolar = Epolar * eVToHartree

      ! ccccc atomic parameters cccccccccccccccccccccccccccccccccccccccc
      ! Experimental (negative) binding energy of Kr 1s electron in eV
      ! (the minus sign is needed)
      E1s = E1s * eVToHartree
      ! Experimental decay width of the hole in eV
      Gamma1s = Gamma1s * eVToHartree
      write (*,*)

      ! Read atomic one-electron eigenenergies (eval) and dipole matrix elements
      ! (SESDatom); these quantities have to be calculated, before running this code,
      ! using oneel
      call readatom(nl, lmin, lmax, nvec, nvlnc, eval, SESdipatm,  &
     &              SESfTfatm, SESpotatm, SESWatom, li, init, Eground)

      write(*,*) 'Number of the S shell of the valence electron, nvlnc =', nvlnc

      ! Recalculate the nocc array after subtraction of the
      ! core orbitals nvlnc
      nshellocc(:) = 0    ! S orbitals are the first orbitals of the periodic table
      nshellocc(1) = 1    ! P orbitals start in the second row of the periodic table
      nshellocc(2) = 2    ! D orbitals start in the third row of the periodic table
      nshellocc(3) = 3    ! F orbitals start in the fourth row of the periodic table
      do l = lmin, lmax
        j = nvlnc - 1 - nshellocc(l)
        if (nocc(l) > 0 .and. j > 0) then
          nocc(l) = nocc(l) - j
          if (nocc(l) < 0) then
            nocc(l) = 0
          end if
        end if
      end do
      write (*,*) 'Occupation pattern in the ground state after subtraction of ',  &
     &  'the core orbitals nvlnc:'
      write (*,*) 'nocc =', (nocc(j), j=lmin, lmax)

      ! Save (radial) transition dipole matrix elements between 1s and np
      ! (n greater or equal 5 in krypton)
      ! No transitions between the initial state and occupied states in
      ! the ground state
      do j = 1, nocc(lf)
        coredip(j) = (0.0D0, 0.0D0)
      end do
      ! Transitions matrix elements between the initial state and virtual
      ! states in the ground state
      do j = nocc(lf)+1, nvec
        coredip(j) = SESdipatm(j,lf,init,li)
      end do

      ! Decouple states that are lower in energy than the valence
      ! state under consideration. Such states are assumed to be fully
      ! occupied. Electrons in the valence shell of angular momentum
      ! "li" must therefore not be allowed to make transitions into
      ! any lower lying levels. (The lowest-energy state in the file
      ! atom.dat is always an s state.)

      ! Core orbitals, including the 1s orbital (which is only singly
      ! occupied after absorbing the x-ray photon), are decoupled in
      ! the Floquet treatment of the Rydberg manifold
      do l = lmin, lmax
        do k = 1, nocc(l)
          do j = lmin, lmax
            do i = 1, nvec
              SESdipatm(i,j,k,l) = (0.0D0, 0.0D0)
              SESdipatm(k,l,i,j) = (0.0D0, 0.0D0)
            end do
          end do
        end do
      end do

      if (fulldiag) then
        allocate(evec(dimen,dimen))
      end if
      Eground = 0.0D0       ! In dreyd not used

      do mf = -lf, lf
        write(*,*) 'Diagonalizing the complex symmetric Hamiltonian ',  &
     &             'with dimension =', dimen, ', lf = ', lf, ', mf = ', mf
        call hamiltonian(dimen, nmain, nphmin, nphmax, lmin, lmax, nvec,  &
     &          mf, cconst, eval, Eground, omegaL, SESfTfatm, SESpotatm,  &
     &          SESWatom, SESdipatm)

        if (fulldiag) then
          jnd = dimen
          call fulldiagfloquet('V', dimen, spectrum(:,mf), evec)
        else
          if (lengthform) then
            call lanczostransfloquet(dimen, nmain, niter, spectrum(:,mf), polestrength(:,mf),  &
     &            nvec, nl, nphmin, lmin, coredip, normsq(mf), jnd)
          else   ! Velocity form
            allocate(lanczstartvec(nvec))
            do i=1, nvec
              lanczstartvec(i) = coredip(i) * (eval(i,lf) - E1s)
            end do
            call lanczostransfloquet(dimen, nmain, niter, spectrum(:,mf), polestrength(:,mf),  &
     &            nvec, nl, nphmin, lmin, lanczstartvec, normsq(mf), jnd)
            deallocate(lanczstartvec)
          end if
        end if

        ! Subtract decay width of 1s core hole from excited states eigenvalues
        do j = 1, jnd
          spectrum(j,mf) = spectrum(j,mf) - cmplx(0.0D0, 0.5D0*Gamma1s, dblprec)
        end do
        ! The full diagonalization does not directly provide
        ! transition matrix elements.  They need to be computed
        ! separately here.
        if (fulldiag) then
          call calctrans(dimen, nphmin, nphmax, lmin, lmax, nvec,  &
     &          lf, evec, coredip, trans(:,mf))
        end if
      end do
      if (fulldiag) then
        deallocate(evec)
      end if

      ! Calculate the total cross section depending on the
      ! x-ray photon energy
      write (*,*) 'Total cross section depending on the x-ray photon energy ',  &
     &          'from Xmin =', Xmin * HartreeToeV, 'to Xmax =', Xmax *          &
     &          HartreeToeV, 'at thetaLX =', thetaLX, 'in nx =', nx, ' steps'
      open(unit=uxsect, file='xsect.dat', status='replace', action='write',  &
     &          form='formatted', access='sequential')
      ! Step size for the calculation of the x-ray photoabsorption cross section
      Delta = (Xmax - Xmin) / real(nx-1, dblprec)
      omegaX = Xmin
      do i = 1, nx
        sigtot = crosssection(omegaX, thetaLX, polarization)
        ! The photoabsorption cross section is calculated in atomic units;
        ! the result is written to the file in kilobarns
        write(uxsect,*) omegaX * HartreeToeV, cross_section_AUtoBarn / 1000.0D0 * sigtot,  &
     &                  polarization * BohrToAngstroem**3
        omegaX = omegaX + Delta
      end do
      close(uxsect)

      ! Calculate the total cross section depending on the
      ! angle between the laser and x-ray polarizations
      write (*,*) 'Total cross section depending on the angle between the ',    &
     &            'laser and x-ray polarizations from angleStart =', angleBeg,  &
     &            'to angleEnd =', angleEnd, 'at Epolar =', Epolar              &
     &            * HartreeToeV, 'in npolar =', npolar, ' steps'
      open(unit=upolar, file='polar.dat', status='replace', action='write',  &
     &          form='formatted', access='sequential')
      ! Step size for the calculation of the x-ray photoabsorption cross section
      Delta = (angleEnd - angleBeg) / real(npolar-1, dblprec)
      thetaLX = angleBeg
      do j = 1, npolar
        sigtot = crosssection(Epolar, thetaLX, polarization)
        ! The absorption cross section is calculated in atomic units;
        ! the result is written to file in kbarn
         write(upolar,*) thetaLX * degtorad, cross_section_AUtoBarn / 1000.0D0 * sigtot,  &
     &                  polarization * BohrToAngstroem**3
         thetaLX = thetaLX + Delta
      end do
      close(upolar)

      deallocate(polestrength, normsq)
      deallocate(SESWatom, SESdipatm, SESpotatm,  SESfTfatm)
      deallocate(eval, coredip, spectrum, trans)

      write (*,*)
      write (*,*) 'Dressed for success [ROXETTE]'


      contains

!
! crosssection --- calculate the sum for the cross section over all
!       dressed states
!
! Note that here the experimental E1s is subtracted!
!

      real(dblprec) function crosssection(omegaX, thetaLX, polarization)
      implicit none

      real(dblprec), intent(in) :: omegaX, thetaLX
      real(dblprec), intent(out) :: polarization
      integer mf, j
      real(dblprec) sigma, alpha, kappa(-lf:lf), a1, a2, p1, p2, avec2
      complex(dblprec) z, z1, z2

      ! Calculate auxiliary quantity kappa_m(thetaLX)
      kappa(-1) = 0.5D0 * sin(thetaLX)**2
      kappa( 0) =         cos(thetaLX)**2
      kappa( 1) = kappa(-1)

      ! Determine photoabsorption cross section
      crosssection = 0.0D0
      polarization = 0.0D0
      p1 = 0.0D0
      p2 = 0.0D0
      do mf = -lf, lf
        sigma = 0.0D0
        alpha = 0.0D0
        a1 = 0.0D0
        a2 = 0.0D0
        if (fulldiag) then
          ! Cross section and polarizability for interaction
          ! Hamiltonian in length form
          do j = 1, dimen
            z1 = trans(j,mf) * trans(j,mf) / (spectrum(j,mf) - omegaX - E1s)
            ! Only the photon absorption part is included in the cross section
            ! not the photon emission part
            sigma = sigma + aimag(z1)
            z2 = trans(j,mf) * trans(j,mf) / (spectrum(j,mf) + omegaX - E1s)
            alpha = alpha + real(z1+z2)
            a1    = a1    + real(z1)
            a2    = a2    + real(z2)
          end do
        ! Lanczos diagonalization
        else
          do j = 1, jnd
            z =     polestrength(j,mf)**2 / (spectrum(j,mf) - omegaX - E1s)
            sigma = sigma + aimag(normsq(mf) * z)
            z = z + polestrength(j,mf)**2 / (spectrum(j,mf) + omegaX - E1s)
            alpha = alpha + real (normsq(mf) * z)
          end do
        end if

        ! Adjustment of the prefactor for the velocity form
        if (.not. lengthform) then
          sigma = sigma / omegaX**2
          polarization = polarization / omegaX**2
          p1           = p1           / omegaX**2
          p2           = p2           / omegaX**2
        end if
        crosssection = crosssection + kappa(mf) * sigma
        polarization = polarization + kappa(mf) * alpha
        p1           = p1           + kappa(mf) * a1
        p2           = p2           + kappa(mf) * a2
      end do

      crosssection = 8.0D0 * M_PI / 3.0D0 * fine_structure_constant * omegaX * crosssection
      polarization = 2.0D0 / 3.0D0 * polarization
      p1 = 2.0D0 / 3.0D0 * p1
      p2 = 2.0D0 / 3.0D0 * p2
      if (omegaX /= 0.0D0) then     
        avec2 = -2.0D0 / omegaX**2 * atmnumZ
      else
        avec2 = 0.0D0	
      end if
      write (183,*) omegaX * HartreeToeV, polarization * BohrToAngstroem**3,  &
     &              p1 * BohrToAngstroem**3, p2 * BohrToAngstroem**3,         &
     &              avec2 * BohrToAngstroem**3
      if (.not. lengthform) then
        polarization = polarization + avec2 / 2.0D0
      end if
      end function crosssection


!
! calctrans --- transformation of the transition dipole moments between
!       a one-particle state in the atomic ground state and all laser
!       dressed atomic states from the finite element basis to the
!       eigenvector basis of the Floquet matrix
!

      subroutine calctrans(dimen,nphmin,nphmax,lmin,lmax,nvec,  &
     &        lf,evec,coredip,trans)
      implicit none

      integer dimen
      integer nphmin,nphmax
      integer lmin,lmax
      integer nvec
      integer lf
      complex(dblprec) evec(dimen,dimen)
      complex(dblprec) coredip(nvec)
      complex(dblprec) trans(dimen)

      integer idx, mu, i, j, l

      do j = 1, dimen
        trans(j) = 0
        idx = 0
        do mu = nphmin, nphmax
          do l = lmin, lmax
            do i = 1, nvec
              idx = idx + 1
              if ((mu .eq. 0) .and. (l .eq. lf)) then
                if (lengthform) then
                  trans(j) = trans(j) + evec(idx,j) * coredip(i)
                else    ! Velocity form
                  trans(j) = trans(j) + evec(idx,j) * coredip(i) * (eval(i,lf) - E1s)
                end if
              end if
            end do
          end do
        end do
      end do
      end subroutine calctrans
      end program dreyd
