!
! giant --- giant resonances
! Christian Buth
! 07 Mar 2007
!
! Version V1.0.0: 20.03.2007 (Xenon 5p ionization is described and
!       neon 1s cross section is reproduced)
!


      program giant
      use mathtl
      use physconst
      use floquetmat
      implicit none

      ! Use full (.true.) or iterative diagonalization (.false.)
      logical fulldiag
      ! Number of x-ray photon energy grid points desired
      integer nLasGrd
      ! Orbital angular momentum quantum numbers for the
      ! initial one-particle state
      integer li
      ! Range of orbital angular momenta taken into account
      ! in Floquet calculation; s, p, and d orbitals should
      ! always be included
      integer lmin, lmax
      ! Occupied orbitals in the s: nocc(0), p: nocc(1), ... shells
      integer, parameter :: maxlocc = 50
      integer nocc(0:maxlocc), nshellocc(0:maxlocc)
      ! Number of l blocks
      integer nl
      ! Range of photon blocks taken into account
      integer nphmin, nphmax
      ! Number of photon blocks
      integer nph
      ! Number of atomic eigenvectors per l and photon block
      integer nvec

      integer dimen, nmain, niter, nvlnc, i, j, k, l, init, jnd, xorder
      integer, parameter :: uxsect = 97

      real(dblprec) LasMin, LasMax, Delta, Gamma, auflux, GRate,  &
     &          sigtot, omegaL, intens, flux, cconst, Eground
      real(dblprec), allocatable :: eval(:,:),     &
     &          polestrength(:,:)


      complex(dblprec), allocatable ::              &
     &          ceval(:,:),                         &
     &          SESWatom(:,:,:),                    &
     &          SESdipatm(:,:,:,:),                 &
     &          SESpotatm(:,:,:),                   &
     &          SESfTfatm(:,:,:),                   &
     &          spectrum(:,:),                      &
     &          evec(:,:)

      namelist/GiantResonance/ nLasGrd, li, xorder,  &
     &          lmin, lmax, nphmin,   &
     &          nphmax, nvec, nocc, flux, LasMin,   &
     &          LasMax, Gamma, init, fulldiag, niter

      ! Initialization
      write (*,*) 'giant V1.1.0, giant resonances'
      write (*,*) '(c) 2007 Christian Buth'
      write (*,*)

      ! Default values for parameters
      nLasGrd = 10001
      li = 2
      lmin = 0 ; lmax = 2
      nl = lmax - lmin + 1
      nphmin = -2 ; nphmax = 2
      nph = nphmax - nphmin + 1
      nvec = 50
      nocc(:) = 0
      xorder = 1
      flux = 1.0D13 / 1.55D0 * eVToHartree / 2.29371257D17
      LasMin = 14310.0D0
      LasMax = 16410.0D0
      Gamma = 0.20D0
      init = 1
      fulldiag = .true.
      niter = 100
      nmain = 1

      read(*,nml=GiantResonance)

      nl = lmax - lmin + 1
      nph = nphmax - nphmin + 1
      if (lmin < 0 .or. lmin > maxlocc .or. lmax < 0 .or. lmax > maxlocc) then
        write (*,*)
        write (*,*) 'giant: illegal parameter range for lmin and lmax'
        write (*,*) 'Range of orbital angular momenta taken into account ', &
     &              'in Floquet calculation;'
        write (*,*) '    s, p, and d orbitals should always be included: lmin =', &
     &              lmin, 'lmax =', lmax
        stop 'giant: illegal parameter range'
      end if

      ! Echo input
      write (*,*) 'Range of orbital angular momenta taken into account ', &
     &            'in Floquet calculation;'
      write (*,*) '    s, p, and d orbitals should always be included: lmin =', &
     &            lmin, 'lmax =', lmax
      write (*,*) 'Number of orbitals occupied in the ground state:'
      write (*,*) 'nocc =', (nocc(j), j=lmin, lmax)
      write (*,*) 'Number of l blocks: nl =', nl
      write (*,*) 'Range of photon blocks taken into account: nphmin =', nphmin, &
     &            'nphmax =', nphmax
      write (*,*) 'Number of photon blocks: nph =', nph
      write (*,*) 'Number of atomic eigenvectors per l and photon block: nvec =', nvec

      dimen = nvec * nl * nph
      write (*,*) 'Dimension of Floquet matrix = ', dimen
      write (*,*) 'Order of multiphoton cross section', xorder
      write (*,*) 'Laser flux in 1/s/cm^2', flux
      write (*,*) 'Examine the photoabsorption of state', init
      write (*,*) 'with angular momentum', li
      write (*,*) 'Experimental decay width of the hole in eV', Gamma
      write (*,*) 'Use full (.true.) or iterative diagonalization (.false.)', fulldiag
      if (.not. fulldiag) then
        write (*,*) 'Carrying out ', niter, ' band-Lanczos iterations',  &
     &              'with ', nmain, 'Lanczos vectors'
      else
        niter = dimen
      end if
      write (*,*) 'Number of laser photon energy grid points: nLasGrd =', nLasGrd
      write (*,*) 'Desired laser photon energy range is from LasMin (in eV)', LasMin,  &
     &            'to LasMax (in eV)', LasMax

      allocate(eval(nvec,lmin:lmax))

      allocate( ceval(nvec,lmin:lmax),                     &
     &          SESWatom(nvec,nvec,lmin:lmax),             &
     &          SESdipatm(nvec,lmin:lmax,nvec,lmin:lmax),  &
     &          SESpotatm(nvec,nvec,lmin:lmax),            &
     &          SESfTfatm(nvec,nvec,lmin:lmax))

      allocate(spectrum(niter,-li:li),       &
     &          polestrength(niter,-li:li))

      ! The desired laser photon energy range is from LasMin (in eV) to
      ! LasMax (in eV) conversion to hartree
      LasMin = LasMin * eVToHartree
      LasMax = LasMax * eVToHartree
      ! Experimental decay width of the hole in eV
      Gamma = Gamma * eVToHartree
      write (*,*)

      ! Read atomic one-electron eigenenergies (eval) and dipole matrix elements
      ! (SESDatom); these quantities have to be calculated, before running this code,
      ! using oneel
      call readatom(nl, lmin, lmax, nvec, nvlnc, eval, SESdipatm,  &
     &              SESfTfatm, SESpotatm, SESWatom, li, init, Eground)
      write(*,*) 'Number of the S shell of the valence electron, nvlnc =', nvlnc

      ! Recalculate the nocc array after subtraction of the
      ! core orbitals nvlnc
      nshellocc(:) = 0    ! S orbitals are the first orbitals of the periodic table
      nshellocc(1) = 1    ! P orbitals start in the second row of the periodic table
      nshellocc(2) = 2    ! D orbitals start in the third row of the periodic table
      nshellocc(3) = 3    ! F orbitals start in the fourth row of the periodic table
      do l = lmin, lmax
        j = nvlnc - 1 - nshellocc(l)
        if (nocc(l) > 0 .and. j > 0) then
          nocc(l) = nocc(l) - j
          if (nocc(l) < 0) then
            nocc(l) = 0
          end if
        end if
      end do
      write (*,*) 'Occupation pattern in the ground state after subtraction of ',  &
     &            'the core orbitals nvlnc:'
      write (*,*) 'nocc =', (nocc(j), j=lmin, lmax)

      ! Decouple states that are lower in energy than the valence
      ! state under consideration. Such states are assumed to be fully
      ! occupied. Electrons in the valence shell of angular momentum
      ! "li" must therefore not be allowed to make transitions into
      ! any lower lying levels. (The lowest-energy state in the file
      ! atom.dat is always an s state.)

      ! Core orbitals, including the 1s orbital (which is only singly
      ! occupied after absorbing the x-ray photon), are decoupled in
      ! the Floquet treatment of the Rydberg manifold

      ! Remove transitions between occupied states
      do l = lmin, lmax
        do k = 1, nocc(l)
          do j = lmin, lmax
            do i = 1, nocc(j)
              SESdipatm(i,j,k,l) = (0.0D0, 0.0D0)
              SESdipatm(k,l,i,j) = (0.0D0, 0.0D0)
            end do
          end do
        end do
      end do

      ! Affix the decay width of the K-vacancy Gamma to the
      ! final state Rydberg orbitals
      do l = lmin, lmax
        do i=1, nvec
          ceval(i,l) = cmplx(eval(i,l), 0.0D0, dblprec)
          if (i > nocc(l)) then
            ceval(i,l) = ceval(i,l) - cmplx(0.0D0, Gamma / 2.0D0, dblprec)
          end if
        end do
      end do

      if (fulldiag) then
        allocate(evec(dimen,dimen))
      end if

      ! Calculate the total cross section depending on the
      ! x-ray photon energy
      write (*,*) 'Total cross section depending on the laser photon energy ',  &
     &          'from LasMin =', LasMin * HartreeToeV, 'to LasMax =', LasMax *  &
     &          HartreeToeV, 'in nLasGrd =', nLasGrd, ' steps'
      write (*,*)
      open(unit=uxsect, file='xsect.dat', status='replace', action='write',  &
     &          form='formatted', access='sequential')
      ! Step size for the calculation of the x-ray photoabsorption cross section
      if (nLasGrd > 1) then
        Delta = (LasMax - LasMin) / real(nLasGrd-1, dblprec)
      else
        write (*,*) 'Just a single(!) point calculation!'
        Delta = 0.0D0
      end if
      omegaL = LasMin
      do i=1, nLasGrd
        write (*,*)
        write (*,*) 'Cross section for omegaL =', omegaL * HartreeToeV, ' eV'
        sigtot = crosssection(omegaL, auflux, GRate)
        ! The photoabsorption cross section is calculated in atomic units;
        ! the result is written to the file in kilobarns
        ! and in (cm^2 s)^xorder / s
        sigtot = sigtot * SecondTot0 * (t0ToSecond*BohrToCentimeter**2)**xorder
        write(uxsect,*) omegaL * HartreeToeV, sigtot, auflux, GRate
        omegaL = omegaL + Delta
      end do
      close(uxsect)

      if (fulldiag) then
        deallocate(evec)
      end if
      deallocate(polestrength, SESWatom, SESdipatm, SESpotatm,  SESfTfatm,  &
     &           eval, spectrum)

      write (*,*)
      write (*,*) 'That''s one small step for man; one GIANT leap for mankind [Neil Armstrong]'


      contains

!
! crosssection --- compute the laser photoabsorption cross section
!       for photons of energy omegaL
!

      real(dblprec) function crosssection(omegaL, auflux, GRate)
      implicit none

      real(dblprec), intent(in) :: omegaL
      integer istart, mf, mu, i, j
      real(dblprec) mind, dis, r, auflux, GRate
      complex(dblprec) closest, z

!       integer, save :: cnt = 100

      ! (LasMin+LasMax) / 2.0: average laser photon energy in eV for a
      !         constant coupling constant over the whole interval,
      !         note that this implies a varying flux!
      ! intens: laser intensity in W/cm^2
      ! conversion to atomic units (an atomic unit of intensity
      ! defined as hartree/t0/a0^2 = 6.43641d15 W/cm^2)
      intens = (LasMin+LasMax) / 2.0D0 * HartreeToJoule * flux * intensity_SItoAU
!      if (intens == 0) then
!        intens = 1.0D-6
!      end if
      write (*,*) 'Flux =', flux, ' photon energy =', omegaL * HartreeToeV,  &
     &            ' intensity =', intens * intensity_AUtoSI
      ! Calculation of the coupling constant
      cconst = sqrt(2.0D0 * M_PI * fine_structure_constant * intens)
      write (*,*) 'laser coupling constant in a.u. = ', cconst

      ! Determine photoabsorption cross section
      GRate = 0.0D0
      do mf = -li, li
        ! Assemble the complex symmetric Hamiltonian
        ! Eground is not subtracted here; 0.0D0 instead
        call hamiltonian(dimen, nmain, nphmin, nphmax, lmin, lmax, nvec,  &
     &          mf, cconst, eval, 0.0D0, omegaL, SESfTfatm, SESpotatm,    &
     &          SESWatom, SESdipatm, ceval)

        ! Diagonalize the complex symmetric Hamiltonian
        if (fulldiag) then
          jnd = dimen
          call fulldiagfloquet('V', dimen, spectrum(:,mf:mf),  &
     &          evec(1:dimen, 1:dimen))

          ! Generate cartesian start vectors.  They depend on the
          ! basis set; the (linear) compound indices are given by
          ! idx -> (mu, l, i) with mu = nphmin..nphmax, l = lmin..
          ! lmax, i = 1..nvec.  Then, the linear index idx, which
          ! corresponds to the triple (mu, l, i) is determined by
          !   idx = nl * nvec * (mu - nphmin) + nvec * (l - lmin) + i
          ! nl : number of angular momenta
          ! 1s state in the mu=0 block
          mu = 0
          l = li
          i = init
          j = 1
          istart = nl * nvec * (mu - nphmin) + nvec * (l - lmin) + i
          write (*,*) 'Lanczos start vector No ', j, ' cartesian index for (mu, l, i) = (',  &
     &                mu, ',', l, ',', i, ') is ', istart

          ! Compute the overlap of the final states with the initial state
          do i=1, jnd
            polestrength(i,mf) = abs(evec(istart,i))**2
          end do

          ! Bubble sort the eigenvalues with ascending real part
          do i = 1, jnd
            do j = i+1, jnd
              if (real(spectrum(i,mf)) > real(spectrum(j,mf))) then
                z = spectrum(i,mf)
                spectrum(i,mf) = spectrum(j,mf)
                spectrum(j,mf) = z
                r = polestrength(i,mf)
                polestrength(i,mf) = polestrength(j,mf)
                polestrength(j,mf) = r
              end if
            end do
          end do
!           cnt = cnt + 1

        else
          stop '!!!!!!! no lanczos implemented'
        end if

        ! Extract the transition rate of the initial state
        ! It is the eigenvalue closest to Eground
        mind = huge(1.0D0)
        do i=1, jnd
          dis = abs(spectrum(i,mf) - Eground)
          if (dis < mind) then
            mind = dis
            closest = spectrum(i,mf)
            r = polestrength(i,mf)
          end if
          ! Output all candidates for the initial state in the light
          if (polestrength(i,mf) > 0.9D0) then
            write (*,*) 'Candidate i=', i, ', mf=', mf, ', polestrength=',  &
     &                  polestrength(i,mf), ', spectrum(i,mf)=', spectrum(i,mf)
          end if
        end do

!         write (*,*) (cnt-1), 'Eigenvalue for m =', mf, ',', closest, ',', r
        write (*,*) 'Eigenvalue for m =', mf, ',', closest, ',', r
        ! The imaginary part of the Siegert energy is
        ! given by -Gamma / 2
        GRate = GRate - 2.0D0 * aimag(closest)
      end do

      ! Add the experimental decay width of the vacancy
      if (intens == 0 .or. omegaL == 0) then
        GRate = 0.0D0
        auflux = 1.0D-10
      else
        auflux = intens / omegaL
      end if

      ! The extra factor of two accounts for the fact that there
      ! are two electrons in each occupied spatial atomic orbital
      crosssection = 2.0D0 * GRate / (auflux)**xorder
      end function crosssection
      end program giant
