!
! zinvit --- inverse iteration to compute an eigenvector of a band matrix
! Thomas Sommerfeld based on a real symmetric code by Hans-Dieter Meyer
! Planet PKS version by Christian Buth
! 6 Sep 2006
!
!
! Copyright (c) 2003 Hans-Dieter Meyer
! Copyright (c) 2003 Thomas Sommerfeld
! Copyright (c) 2006 Christian Buth
!
! This library is free software; you can redistribute it and/or
! modify it under the terms of the GNU Lesser General Public
! License as published by the Free Software Foundation; either
! version 2.1 of the License, or (at your option) any later version.
!
! This library is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
! Lesser General Public License for more details.
!
! You should have received a copy of the GNU Lesser General Public
! License along with this library; if not, write to the Free Software
! Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
!
!
c
c     inverse iteration to compute an eigenvector of a band matrix
c     plus dependencies:
c       zbandec
c       zbanbks
c       znorm
c       znormalize
c
c     the corresponding real routines are from numerical recipies
c
c
      subroutine zinvit(ndim,np,np1,mm,ew,t,a,a1,b,ipvt,ninvi,debug)
      implicit none
c
      integer ndim, np, np1, mm, ipvt(ndim), ninvi
      complex*16 t(np1,ndim), a(ndim,mm), a1(ndim,mm), b(ndim), ew
      logical debug
c
      complex*16 y
      real*8 seed, dum
c
      parameter (seed = 1.2345678904711)
c
      integer i, j
c     --------------------------------------------
c     t nach a in compact storage mode
c     --------------------------------
      do j = 1, mm
         do i = 1, ndim
            a(i,j) = (0.0d0, 0.0d0)
         enddo
      enddo
c
      do i = 1, ndim
         a(i,np1) = t(1,i)
      enddo
      do j = 1, np
         do i = 1, ndim - j
            a(i,np1+j) = t(j+1,i)
            a(i+j,np1-j) = t(j+1,i)
         enddo
      enddo
c     ------------------------
c     A - ew bilden
c     -------------
      do i = 1, ndim
         a(i,np1) = a(i,np1) - ew
      enddo
c     -----------------------------------------
c     LU Decomposition of a - ew
c     --------------------------
      call zbandec(a,ndim,np,np,ndim,mm,a1,mm,ipvt,dum)
c     ---------------------------
c     invers Iteration
c     b has been prepared in banddia
c     ----------------
c      call durand(seed, 2*ndim, b)
c
      write(6,100) ninvi, ew
 100  format(/,'  Doing ',I2,' inverse iterations for EV = (',F14.8
     $     ,',',F14.8,')')
c
      do j = 1, ninvi
         call zbanbks(a,ndim,np,np,ndim,mm,a1,mm,ipvt,b)
         call znorm(ndim, b, y)
         if (debug) write(6,110) j, y
 110     format('  Norm(',I2,') = (',E9.2,',',E9.2,')')
      enddo
      call znormalize(ndim, b, y)
      if (debug) then
         do i = 1, ndim
            write(6,200) i, b(i)
 200        format ('x(',I2,') = (',F9.5,',',F9.5,')')
         enddo
      endif
c
      return
      end
c
c     #################################################
c
      subroutine zbandec(a,n,m1,m2,np,mp,a1,mp1,indx,d)
c     ------------------------------------------------
c     LU decomposition of a band martix a
c     numerical recipes --> chapter 2, banddiagonal systems
c     a is destroyed and contatins the upper triangle
c     a1 contains the lower triangle
c     indx records the row permutations
c     -----------------------------------------------------
      implicit none
c
      integer m1, m2, mp, mp1, n, np, indx(n)
      real*8 tiny, d
      complex*16 a(np,mp), a1(np,mp1)
c
      parameter (tiny = 1.d-20)
c
      integer i, j, k, l, mm
      complex*16 dum
c
      mm = m1 + m2 + 1
      if (mm.gt.mp .or. m1.gt.mp1 .or. n.gt.np) then
         write(6,*) 'bad args in zbandec'
         stop
      endif
c
      l = m1
      do 13 i = 1, m1
         do 11 j = m1 + 2 - i, mm
            a(i,j-l) = a(i,j)
 11      continue
         l = l - 1
         do 12 j = mm - l, mm
            a(i,j) = (0.0d0, 0.0d0)
 12      continue
 13   continue
c
      d = 1.0d0
      l = m1
c
c     for each row ...
c
      do 18 k = 1, n
         dum = a(k,1)
         i = k
         if (l .lt. n) then
            l = l + 1
         endif
c
c     Find the pivot
c
         do 14 j = k+1, l
            if (zabs(a(j,1)) .gt. zabs(dum)) then
               dum = a(j,1)
               i = j
            endif
 14      continue
c
         indx(k) = i
         if (zabs(dum) .eq. 0.0d0) then
            a(k,1) = dcmplx(tiny, 0.0d0)
         endif
c
         if (i .ne. k) then
c     Interchange rows
            d = -d
            do 15 j = 1, mm
               dum = a(k,j)
               a(k,j) = a(i,j)
               a(i,j) = dum
 15         continue
         endif
c
c     Do the elimination
c
         do 17 i = k + 1, l
            dum = a(i,1) / a(k,1)
            a1(k,i-k) = dum
            do 16 j = 2, mm
               a(i,j-1) = a(i,j) - dum*a(k,j)
 16         continue
            a(i,mm) = (0.0d0, 0.0d0)
 17      continue
c
 18   continue
c
      return
      end
c
c     #######################################################
c
      subroutine zbanbks(a,n,m1,m2,np,mp,a1,mp1,indx,b)
c     ------------------------------------------------------
c     solves Ax=b for x
c     A is a bandmatrix and has been decomposed in upper(a)
c     and lower(a1) triangle with bandec.
c     x is returned in b
c     ------------------------------------------------------
      implicit none
c
      integer m1, m2, mp, mp1, n, np, indx(n)
      complex*16 a(np,mp), a1(np,mp1), b(n)
c
      integer i, k, l, mm
      complex*16 dum
c
      mm = m1 + m2 + 1
      if (mm.gt.mp .or. m1.gt.mp1 .or. n.gt.np) then
         write(6,*) 'bad args in banbks'
         stop
      endif
c
c     Forward substitution unscrambling permuted rows
c
      l = m1
      do 12 k = 1, n
         i = indx(k)
         if (i .ne. k) then
            dum = b(k)
            b(k) = b(i)
            b(i) = dum
         endif
         if (l .lt. n) then
            l = l + 1
         endif
         do 11 i = k + 1, l
            b(i) = b(i) - a1(k,i-k)*b(k)
 11      continue
 12   continue
c
c     Backsubstitution
c
      l = 1
      do 14 i = n, 1, -1
         dum = b(i)
         do 13 k = 2, l
            dum = dum - a(i,k)*b(k+i-1)
 13      continue
         b(i) = dum/a(i,1)
         if (l .lt. mm) then
            l = l + 1
         endif
 14   continue
c
      return
      end
c
c     ######################################
c
      subroutine znorm (ndim, b, norm)
c     ------------------------------------
c     b ist der Vektor, bleibt unveraendert
c     ndim seine Dimension
c     norm gibt die Norm zurueck
c     ------------------------------------
      implicit none
c
      integer ndim
      complex*16 b(ndim), norm
c
      integer i
c
      norm = (0.0d0, 0.0d0)
      do 10 i = 1, ndim
         norm = norm + b(i)**2
 10   continue
c
      return
      end
c
c     #######################################
c
      subroutine znormalize (ndim, b, norm)
c     ------------------------------------
c     b ist der zu normierende Vektor
c       ndim seine Dimension
c     norm gibt die Norm zurueck
c     ------------------------------------
      implicit none
c
      integer ndim
      complex*16 b(ndim), norm
c
      integer i
      complex*16 y
c
      norm = (0.0d0, 0.0d0)
      do 10 i = 1, ndim
         norm = norm + b(i)**2
 10   continue
c
      y = (1.0d0, 0.0d0) / zsqrt(norm)
      do 20 i = 1, ndim
         b(i) = b(i) * y
 20   continue

      end
