!
! SESWmtrx --- Calculate matrix representation of the CAP operator W
!       derived from smooth exterior complex scaling (SES)
! Christian Buth
! 17 May 2006
!


!
! The subroutine calculates the matrix of the CAP operator W(r) in eigenfunction
! representation, r being the radial coordinate.  The integrals are evaluated
! upon the substitution x=sqrt(r) (r is distributed on a quadratic mesh,
! x on a linear one). xh is the spacing of the linear mesh.
!
! ngrd  : number of grid points of the quadratic mesh
! xh    : spacing of points in the linear x=sqrt(r)-mesh
! egfxn : eigenfunctions of the atomic Hamiltonian with the
!         Herman-Skillman potential represented
!         in the finite element basis
! theta : complex scaling angle
!

      subroutine SESWmtrx(Wbig, nstate, nl, egfxn, ngrd, xh)
      use quadrature
      use SESpath
      implicit none

      integer nstate, nl, ngrd
      integer i, j, k, l
      real(dblprec) egfxn(ngrd,nstate,nl)
      real(dblprec), allocatable :: rework(:), imwork(:)
      real(dblprec) r, x, xh, numinterr, re1, im1, re2, im2
      complex(dblprec) Wbig(nstate,nstate,nl)
      complex(dblprec) value
!       real(dblprec) rint
!       external rint

      allocate(rework(ngrd), imwork(ngrd))
      numinterr = 0.0D0

      ! Sample W_SES on the numeric (quadratic) integration grid
      do l = 1, nl
        do k = 1, nstate
          do j = 1, k
            do i = 1, ngrd
              ! Evaluate W_SES at x
              x = xh * real(i, dblprec)
              rework(i) = egfxn(i,j,l) * egfxn(i,k,l)           ! Eigenfunction values
              rework(i) = 2.0D0 * x * rework(i)                 ! Integration measure
              imwork(i) = rework(i)
              value = ((-1.0D0,0.0D0) / (8.0D0,0.0D0)) * ((2.0D0, 0.0D0) &
     &                * pathfpp(x**2) * pathf(x**2) &
     &                - (3.0D0,0.0D0) * pathfp(x**2)**2) / pathf(x**2)**4

              ! Eliminate spurious contamination
!               if (x**2 < sesrlow) then
!                 value = (0.0D0, 0.0D0)
!               end if
!               if (x**2 < sesrlow .and. abs(value) > 1.0D-8) then
!                 write (*,*) j, k, l, x**2, abs(value)
!               end if
              rework(i) = rework(i) * real (value, dblprec)
              imwork(i) = imwork(i) * aimag(value)
            end do

            ! Numerical integration
!             re1 = midpointrule (ngrd,xh,rework)
!             im1 = midpointrule (ngrd,xh,imwork)
!             re1 = trapezoidrule(ngrd,xh,rework)
!             im1 = trapezoidrule(ngrd,xh,imwork)
!             re2 = simpsonsrule (ngrd,xh,rework)
!             im2 = simpsonsrule (ngrd,xh,imwork)
            re1 = threeeighthsrule(ngrd,xh,rework)
            im1 = threeeighthsrule(ngrd,xh,imwork)
            re2 = simpsonsrule    (ngrd,xh,rework)
            im2 = simpsonsrule    (ngrd,xh,imwork)
            r = (re1 - re2)**2 + (im1 - im2)**2
            if (r > numinterr) then
              numinterr = r
            end if
            Wbig(j,k,l) = cmplx(re2, im2, dblprec)
            Wbig(k,j,l) = Wbig(j,k,l)                       ! Wbig is complex symmetric
          end do
        end do
      end do

      write (*,*) '        Maximum error in numerical integration ', sqrt(numinterr)
      deallocate(rework, imwork)
      end subroutine SESWmtrx
