!
! pulseprop --- propagate two pulses of different color through a
!       medium
! Christian Buth, Robin Santra
! 02 Nov 2007 (Taken over from Robin)
!
! Version 1.0.0: 02 Nov 2007 (Propagation of laser and x-ray pulses
!       through neon)
!


!
! Output into FORTRAN units:
! fort.20: x-ray pulse profile (photon flux)
! fort.2?: x-ray and laser pulses for various steps in the pulse propagation
! fort.50: initial plot of the number of atoms in the gas cell
! fort.55: phase and group velocities of the laser, x rays in the plasma and the
!       argon gas
! fort.92: intensity in W/cm^2 and cross section in kilobarns
! fort.97: number of ground-state atoms and ions in the gas
!


      program pulseprop
      use mathtl
      use physconst
      implicit none

!       integer, parameter :: ngrid = 1001,  &   ! For argon
      integer, parameter :: ngrid = 100001,  &   ! For argon
!      &                      ngrid = 10001,  &   ! For neon
     &                      ndata = 12          ! For argon
!     &                      ndata = 9          ! For neon
      real(dblprec) Jxnew(ngrid),Jxold(ngrid),nGS(ngrid),nION(ngrid),z(ngrid)
      real(dblprec) iL(ndata),sigX(ndata),b(ndata),c(ndata),d(ndata)
      real(dblprec) nAT, pgas, TRoom
      real(dblprec) zmin,zmax,L,dz,dt,z0,t0
      real(dblprec) nph,Delta
      real(dblprec) tauX,tauL
      real(dblprec) zX, zL
      real(dblprec) I0, LasInt
      real(dblprec) pulseX,pulseL
      real(dblprec) sigma,aux
      real(dblprec) nEL, wP, wX, wL, derpolmaxL, derpolmaxX,       &
     &          polmaxL, polmaxX, vgGL, vgGX, vgplL, vgplX, vplL,  &
     &          vplX, vpGL, vpGX, nPLmo, nPXmo, nPLmoPrev,         &
     &          nPXmoPrev, vplLPrev, vplXPrev, vgplLPrev,          &
     &          vgplXPrev, vpGLPrev, vpGXPrev, vgGLPrev, vgGXPrev

      real(dblprec), parameter :: c_cm_ps = SpeedOfLight * 1.0D-10
      integer i,j

      namelist/PulsePropagation/ LasInt

      common /sigpar/ iL,sigX,b,c,d

      ! Initialization
      write (*,*) 'pulseprop V1.0.1, laser and x-ray pulse propagation'
      write (*,*) '(c) 2007 Christian Buth, Robin Santra'
      write (*,*)

      ! Default values for parameters
      ! Gas pressure in atmospheres
      Pgas = 5.0D0
      Pgas = Pgas * StdAtmosphere
      ! Room temperature in Kelvin
      TRoom = 300.0D0
      ! nAT: density of neutral atoms in gas cell, in 1/cm^3. This is
      ! the density before ionization by the x-ray pulse
!     nAT = 2.4D19            ! Neon
      nAT = Pgas / Troom / Boltzmann     ! Argon
      nAT = nAT / (1.0D2)**3
      write (*,*) 'Number density ', nAT, 'cm^-3'

!     L: length of gas cell, in cm.
!     It is assumed that the gas cell starts at z=0 and ends at z=L
!       L = 0.2D0                 ! Neon
      L = 0.6D0                 ! Argon

!     zmin: smallest z considered, in cm. zmin should be negative
!       zmin = -10.0D0          ! Neon
      zmin = -10.0D0            ! Argon

!     zmax: largest z considered, in cm.  zmax should be greater than L
!       zmax = 10.0D0          ! Neon
      zmax = 10.0D0            ! Argon

!     dz: spatial step size, in cm
      dz = (zmax - zmin) / real(ngrid-1, dblprec)
      write (*,*) 'Spatial step size', dz, 'cm'

!     dt: temporal step size, in ps.  dt = dz/c
      dt = dz / c_cm_ps
      write (*,*) 'Temporal step size', dt, 'ps'

!     z0: initial position of x-ray/laser pulse maximum, in cm.
!     z0 should be negative, such that both pulses are entirely
!     outside (to the left of) the gas cell
!       z0 = -5.0D0       ! Neon
      z0 = -5.0D0       ! Argon

!     wX: x-ray photon energy, in eV
!       wX = 867.36d0           ! Neon
      wX = 3203.42D0           ! Argon
!     conversion to hartree
      wX = wX * eVToHartree

!     nph: number of photons in x-ray pulse
      nph = 1.0D6

!     Delta: x-ray focal width (FWHM), in cm
      Delta = 1.0D-3

!     tauX: duration (FWHM) of x-ray pulse, in ps
      tauX = 100.0D0
!     conversion to length, in cm. zX = c*tauX
      zX = c_cm_ps * tauX
      write (*,*) 'Spatial x-ray FWHM size', zX, 'cm'

!     wL: laser photon energy, in eV
!       wL = 1.55D0             ! Neon
      wL = 1.5498D0           ! Argon
!     conversion to hartree
!       wL = wL/27.2114d0
      wL = wL * eVToHartree

!     tauL: duration (FWHM) of laser pulse, in ps
!       tauL = 3.0D0              ! Neon
      tauL = 1.0D0              ! Argon
!     conversion to length, in cm. zL = c*tauL
      zL = c_cm_ps * tauL
      write (*,*) 'Spatial laser FWHM size', zL, 'cm'

!     I0: peak laser intensity, in 10^13 W/cm^2
      LasInt = 1.0D13

      read(*,nml=PulsePropagation)

      write (*,*) 'Peak laser intensity ', LasInt, 'W/cm^2'
      I0 = LasInt
      I0 = I0 / 1.0D13

      ! Read in x-ray absorption cross section, in kb, as a
      ! function of laser intensity, in W/cm^2
      open(unit=10, file='sigma.dat', access='sequential', status='old',  &
     &     form='formatted')
      do i = 1, ndata
        read(10,*) iL(i), sigX(i)
        ! Keep numbers small
        iL(i) = iL(i) / 1.0D13
        ! Conversion of cross section from kb to cm^2
        sigX(i) = 1.0D-21 * sigX(i)
      end do
      close(unit=10)

      call spline(ndata,iL,sigX,b,c,d)
      t0 = 0.0D0

      ! Construct spatial grid
      z(1) = zmin
      do i = 2, ngrid
        z(i) = z(i-1) + dz
      end do

      ! Initial atomic and ionic density in the gas cell
      do i = 1, ngrid
        if ((z(i) >= 0.0D0) .and. (z(i) <= L)) then
          nGS(i) = nAT
        else
          nGS(i) = 0.0D0
        end if
        nION(i) = 0.0D0
      end do

      do i = 1, ngrid
        write(50,*) z(i), nGS(i) / 1.0D18
      end do

      ! Discretize initial x-ray pulse; the laser pulse
      ! stays always the same
      ! X-ray pulse profile (photon flux)
      aux = 2.0D0 * sqrt(log(2.0D0) / M_PI) * nph / tauX
      ! Rate to flux conversion
      aux = 4.0D0 * log(2.0D0) / M_PI * aux / Delta**2
      write (*,*) 'Peak x-ray flux', aux, '(ps cm^2)^-1'

      ! Write out initial laser and x-ray pulses
      do i = 1, ngrid
        Jxold(i) = pulseX(z(i),z0,zX,aux)
        write(20,*) z(i), Jxold(i), pulseL(z(i), z0, zL, I0) * 1.0D13
      end do

      ! Write out intensity in W/cm^2 and cross section in kilobarns
      aux = 0.0D0
      do i = 1, ngrid
        write(92,*) aux * 1.0D13, sigma(aux) * 1.0D21
        aux = aux + 1.0D-4
      end do

      ! Propagation of the pulses over the grid
      do j = 1, ngrid
        t0 = t0 + dt
        z0 = z0 + dz
        Jxnew(1) = 0.0D0                ! Fill the left-most cell with zero

        ! Reduction of the x-ray flux due to absorption
        do i = 2, ngrid
          aux = pulseL(z(i-1),z0,zL,I0)
          aux = nGS(i-1) * sigma(aux) * dz
          aux = 1.0D0 - aux
          Jxnew(i) = aux * Jxold(i-1)
        end do

        ! Number of ions created due to photoabsorption
        do i = 1, ngrid
          if ((z(i) >= 0.0D0) .and. (z(i) <= L)) then
            aux = pulseL(z(i),z0,zL,I0)
            aux = sigma(aux) * Jxold(i) * dt
            aux = aux * nGS(i)
            nGS (i) = nGS (i) - aux
            nION(i) = nION(i) + aux
          end if
        end do

        ! Write snapshots of the pulse propagation
!        if (j.eq.2437) then    ! Neon
        ! Before the gas cell
!         if (j == int(5.0D0 / dz)) then
!         if (abs(z0 - 5.0D0) < 1.0D-10) then
!           write (*,*) 'Grid position -10cm, ie.,', 5.0D0 / dz, 'time steps'
!           do i = 1, ngrid
!              if (mod(i,10) == 0) then
!                write(20,*) z(i), Jxnew(i), pulseL(z(i),z0,zL,I0) * 1.0D13
!              end if
!           end do
!         end if
        ! Right in the gas cell
!         if (j == int(15.0D0 / dz)) then
        if (abs(z0 - 0.0D0) < 1.0D-10) then
          write (*,*) 'Grid position 0cm, ie.,', 10.0D0 / dz, 'time steps'
          do i = 1, ngrid
             if (mod(i,10) == 0) then
               write(21,*) z(i), Jxnew(i), pulseL(z(i),z0,zL,I0) * 1.0D13
             end if
          end do
        end if
        ! In and after the gas cell
!         if (j == int(17.5D0 / dz)) then
        if (abs(z0 - 2.5D0) < 1.0D-10) then
          write (*,*) 'Grid position 2.5cm, ie.,', 12.5D0 / dz, 'time steps'
          do i = 1, ngrid
             if (mod(i,10) == 0) then
               write(22,*) z(i), Jxnew(i), pulseL(z(i),z0,zL,I0) * 1.0D13
             end if
          end do
        end if
        ! After the gas cell
!         if (j == int(17.5D0 / dz)) then
        if (abs(z0 - 5.0D0) < 1.0D-10) then
          write (*,*) 'Grid position 5cm, ie.,', 15.0D0 / dz, 'time steps'
          do i = 1, ngrid
             if (mod(i,10) == 0) then
               write(23,*) z(i), Jxnew(i), pulseL(z(i),z0,zL,I0) * 1.0D13
             end if
          end do
        end if

        Jxold(:) = Jxnew(:)
      end do

      ! Output grid after pulse has gone through the gas cell
      write (*,*)
      nPLmoPrev = 0.0D0
      nPXmoPrev = 0.0D0
      vplLPrev  = 0.0D0
      vplXPrev  = 0.0D0
      vgplLPrev = 0.0D0
      vgplXPrev = 0.0D0
      vpGLPrev  = 0.0D0
      vpGXPrev  = 0.0D0
      vgGLPrev  = 0.0D0
      vgGXPrev  = 0.0D0

      do i = 1, ngrid
        ! Local electron density, in cm^-3
        ! factor of two due to Auger decay of core hole
        nEL = 2.0D0 * nION(i)
        ! Conversion to atomic units.
!         nEL = nEL * (0.52918d-8)**3
        nEL = nEL / CentimeterToBohr**3
        ! Local plasma frequency, in atomic units
!         wP = sqrt(12.566370614D0 * nEL)
        wP = sqrt(4.0D0 * M_PI * nEL)

        ! Index of refraction of the laser in the plasma
!         nP = sqrt(1.0D0 - wP**2 / wL**2)
!         nPL = 1.0D0 - 0.5D0 * wP**2 / wL**2
        nPLmo = -0.5D0 * wP**2 / wL**2
        ! Index of refraction of x rays in the plasma
!         nP = sqrt(1.0D0 - wP**2 / wX**2)
!         nPX = 1.0D0 - 0.5D0 * wP**2 / wX**2
        nPXmo = -0.5D0 * wP**2 / wX**2
        ! Local phase velocity of the laser in the plasma
!         vplL = SpeedOfLight_AU / nPL
!         vplL = vplL / SpeedOfLight_AU - 1.0D0
        vplL = -nPLmo
        ! Local phase velocity of x rays in the plasma
!         vplX = SpeedOfLight_AU / nPX
!         vplX = vplX / SpeedOfLight_AU - 1.0D0
        vplX = -nPXmo
        ! Local group velocity of the laser in the plasma
!         vgplL = SpeedOfLight_AU * nPL
!         vgplL = vgplL / SpeedOfLight_AU - 1.0D0
        vgplL = nPLmo
        ! Local group velocity of x rays in the plasma
!         vgplX = SpeedOfLight_AU * nPX
!         vgplX = vgplX / SpeedOfLight_AU - 1.0D0
        vgplX = nPXmo

!         ! Local group velocity of x rays, in cm/ps
!         vX = (wP/wX)**2
!         vX = sqrt(1.0D0 + vX)
!         vX = c_cm_ps/vX
!         ! Local group velocity of laser, in cm/ps
!         vL = (wP/wL)**2
!         vL = sqrt(1.0D0 + vL)
!         vL = c_cm_ps/vL


!
! From Ref. Rahman:DD-90:
! omega [a.u.]          alpha(omega) [a.u.]
! 0.05                  11.3881
! 0.057                 11.4204
! 0.1                   11.6190
!
! Linear interpolation:
! m = (11.6190 - 11.3881) / (0.1 - 0.05) = 4.618
! b = 11.6190 - m * 0.1 = 11.1572
! m * (0.057 - 0.1) + 11.6190 = 11.420426 = m * 0.057 + b
!

        ! Maximum derivative of the polarizability of argon in laser light
        derpolmaxL = (11.6190D0 - 11.3881D0) / (0.1D0 - 0.05D0)
        if (i == 1) then
          write (*,*) 'Maximum derivative of the polarizability ',  &
     &                'in laser light', derpolmaxL * BohrToAngstroem**3 / HartreeToeV
        end if
        ! Maximum polarizability of argon in laser light
        polmaxL = derpolmaxL * (0.057D0 - 0.1D0) + 11.6190D0
        if (i == 1) then
          write (*,*) 'Maximum polarizability in laser light',  &
     &                polmaxL * BohrToAngstroem**3
        end if
        derpolmaxL = derpolmaxL * BohrToCentimeter**3
        polmaxL    = polmaxL    * BohrToCentimeter**3
        ! Phase velocity of x rays in neutral argon gas
!         vpGL = SpeedOfLight_AU / (1.0D0 + 2.0D0 * M_PI * nGS(i) * polmaxL)
!         vpGL = vpGL / SpeedOfLight_AU - 1.0D0
        vpGL = -2.0D0 * M_PI * nGS(i) * polmaxL
        ! Group velocity of x rays in neutral argon gas
!         vgGL = SpeedOfLight_AU / (1.0D0 + 2.0D0 * M_PI * nGS(i)  &
!      &         * (polmaxL + wL * derpolmaxL))
!         vgGL = vgGL / SpeedOfLight_AU - 1.0D0
        vgGL = -2.0D0 * M_PI * nGS(i) * (polmaxL + wL * derpolmaxL)
        ! Maximum derivative of the polarizability of argon on resonance
        derpolmaxX = derpolmaxL
        if (i == 1) then
          write (*,*) 'Maximum derivative of the polarizability ',  &
     &                'in x-ray light', derpolmaxX * BohrToAngstroem**3 / HartreeToeV
        end if
        ! Maximum polarizability of argon for x rays
        polmaxX = polmaxL
        if (i == 1) then
          write (*,*) 'Maximum polarizability in x-ray light',  &
     &                polmaxX * BohrToAngstroem**3
        end if
        derpolmaxX = derpolmaxX * BohrToCentimeter**3
        polmaxX    = polmaxX    * BohrToCentimeter**3
        ! Phase velocity of x rays in neutral argon gas
!         vpGX = SpeedOfLight_AU / (1.0D0 + 2.0D0 * M_PI * nGS(i) * polmaxX)
!         vpGX = vpGX / SpeedOfLight_AU - 1.0D0
        vpGX = -2.0D0 * M_PI * nGS(i) * polmaxX
        ! Group velocity of x rays in neutral argon gas
!         vgGX = SpeedOfLight_AU / (1.0D0 + 2.0D0 * M_PI * nGS(i)  &
!      &         * (polmaxX + wX * derpolmaxX))
!         vgGX = vgGX / SpeedOfLight_AU - 1.0D0
        vgGX = -2.0D0 * M_PI * nGS(i) * (polmaxX + wX * derpolmaxX)

        ! Write data to units
        write(97,*) z(i), nION(i), nGS(i)
        write(55,*) z(i), nPLmo, nPXmo, vplL, vplX, vgplL, vgplX,  &
     &             vpGL, vpGX, vgGL, vgGX

        ! Determine maxima/minima of the quantities
        if (abs(nPLmo) > abs(nPLmoPrev)) then
          nPLmoPrev = nPLmo
        end if
        if (abs(nPXmo) > abs(nPXmoPrev)) then
          nPXmoPrev = nPXmo
        end if
        if (abs(vplL) > abs(vplLPrev)) then
          vplLPrev = vplL
        end if
        if (abs(vplX) > abs(vplXPrev)) then
          vplXPrev = vplX
        end if
        if (abs(vgplL) > abs(vgplLPrev)) then
          vgplLPrev = vgplL
        end if
        if (abs(vgplX) > abs(vgplXPrev)) then
          vgplXPrev = vgplX
        end if
        if (abs(vpGL) > abs(vpGLPrev)) then
          vpGLPrev = vpGL
        end if
        if (abs(vpGX) > abs(vpGXPrev)) then
          vpGXPrev = vpGX
        end if
        if (abs(vgGL) > abs(vgGLPrev)) then
          vgGLPrev = vgGL
        end if
        if (abs(vgGX) > abs(vgGXPrev)) then
          vgGXPrev = vgGX
        end if
      end do

      write (*,*)
      write (*,*) 'Maximum plasma index of refraction for the laser', nPLmoPrev
      write (*,*) '        and the x rays', nPXmoPrev
      write (*,*) 'Maximum phase velocity of the laser in the plasma', vplLPrev
      write (*,*) '        and the x rays', vplXPrev
      write (*,*) 'Maximum group velocity of the laser in the plasma', vgplLPrev
      write (*,*) '        and the x rays', vgplXPrev
      write (*,*) 'Maximum phase velocity of the laser in the gas', vpGLPrev
      write (*,*) '        and the x rays', vpGXPrev
      write (*,*) 'Maximum group velocity of the laser in the gas', vgGLPrev
      write (*,*) '        and the x rays', vgGXPrev
      end program pulseprop


!
! pulseX --- the x-ray pulse
!

      function pulseX(z,z0,zL,J0)
      use fortrantl
      implicit none

      real(dblprec) pulseX, z, z0, zL, J0
      real(dblprec), parameter :: aux = 4.0D0 * log(2.0D0)

      pulseX = J0 * exp(-aux*((z-z0)/zL)**2)
      end function pulseX


!
! pulseL --- the laser pulse
!

      function pulseL(z,z0,zL,I0)
      use fortrantl
      use mathtl
      implicit none

      integer i
      real(dblprec) pulseL,z,z0,zL,I0
      real(dblprec), parameter :: aux = 4.0D0 * log(2.0D0)

!       ! Neon pulse
!       pulseL = exp(-aux*((z-z0-2.5D0*zL)/zL)**2)
!       pulseL = pulseL + exp(-aux*((z-z0+2.5D0*zL)/zL)**2)
!       pulseL = I0 * pulseL

      ! Argon pulse
      pulseL = 0.0D0
      do i=0, 4
        pulseL = pulseL + exp(-aux*((z-z0 + 2.0D0 * (real(i, dblprec) - 2.0D0) * zL)/zL)**2)  &
     &                  * cos(M_PI / (2.0D0*8.0D0) * real(i, dblprec))
      end do
      pulseL = I0 * pulseL
      end function pulseL


!
! sigma --- the x-ray absorption cross section
!

      function sigma(intens)
      use fortrantl
      implicit none

      real(dblprec) sigma,intens
      integer ndata
      parameter (ndata = 12)
!       parameter (ndata = 9)
      real(dblprec) iL(ndata),sigX(ndata),b(ndata),c(ndata),d(ndata)
      external seval
      real(dblprec) seval
!      real(dblprec) a0,a1,a2,a3,a4,a5,a6,a7,x

      common /sigpar/ iL,sigX,b,c,d

      sigma = seval(ndata,intens,iL,sigX,b,c,d)

      ! Neon cross section
!!$      a0 = 7.84824d+11
!!$      a1 = 1.58002d+12
!!$      a2 = 7.9314d+13
!!$      a3 = 1.77828d+13
!!$      a4 = 5.13404d+08
!!$      a5 = 1.14381d+10
!!$      a6 = 2.70077d+11
!!$      a7 = 5.951d+11
!!$
!!$      x = intens
!!$
!!$      sigma = a0 + a1*x + a2*x**2 + a3*x**3
!!$      sigma = sigma/(a4 + a5*x + a6*x**2 + a7*x**3)
!!$      sigma = sigma*1.d-21
      end function sigma
